{% extends "podcast_analyzer/app_base.html" %}
{% load i18n %}
{% block extratitle %}{% translate "People" %} - {% endblock %}

{% block breadcrumbs %}

  <li class="is-active"><a href="#" aria-current="page">{% translate "People" %}</a></li>
{% endblock breadcrumbs %}

{% block content %}
  <h1>{% translate "People" %}</h1>
  <ul>
    {% for person in page_obj %}
      <li>
        <article>
          <figure>
            <p>
              <a href="{{ person.get_absolute_url }}">{% if person.img_url %}<img src="{% if person.avatar %}{{ person.avatar.url }}{% else %}{{ person.img_url }}{% endif %}" alt="Avatar for {{ person.name }}"  width="200" height="200"/> {% endif %}{{ person.name }}</a>
            </p>
          </figure>
        </article>
      </li>
    {% empty %}
      <li>{% translate "There are no people yet." %}</li>
    {% endfor %}
  </ul>

  {% include "podcast_analyzer/includes/pagination.html" %}

{% endblock content %}
