{% extends "base.html" %}
{% load i18n %}
{% block app_breadcrumb %}
  <li><a href="{% url 'podcast_analyzer:entry' %}">{% translate "Analyzer" %}</a></li>
{% endblock app_breadcrumb %}

{% block content %}
{% endblock content %}
