{% load i18n static %}<!DOCTYPE html>
{% get_current_language as LANGUAGE_CODE %}
<html lang="{{ LANGUAGE_CODE }}">
  <head>
    <meta charset="utf-8" />
    <meta http-equiv="x-ua-compatible" content="ie=edge"/>
    <title>{% block extratitle %}{% endblock %}{% translate "Podcast Tracker" %}</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="description" content="A tool for tracking given podcasts and analyzing the publicly available feed data." />
    <meta name="author" content="Daniel Andrlik"/>
    {% block css %}
    {% endblock css %}
    {% block javascript %}
    {% endblock javascript %}
  </head>
  <body>
    <section>
      <div>
        <section>
          <nav>
            <div>
              <div>
                <nav aria-label="breadcrumbs">
                  <ul>
                    {% block app_breadcrumb %}{% endblock %}
                    {% block breadcrumbs %}
                    {% endblock breadcrumbs %}
                  </ul>
                </nav>
              </div>
            </div>
            <div>
              {% block extra_controls %}{% endblock %}
            </div>
          </nav>
        </section>
        {% if messages %}
          <div>
            <ul>
              {% for message in messages %}<li {% if message.tags %}class="{{ message.tags }}"{% endif %}>{{ message }}</li>{% endfor %}
            </ul>
          </div>
        {% endif %}
        {% block content %}{% endblock %}
      </div>
    </section>
    {% block extrajs %}{% endblock %}
  </body>
</html>
