# Generated by Django 5.1.2 on 2024-10-17 19:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("podcast_analyzer", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="person",
            name="img_url",
            field=models.URLField(
                blank=True, help_text="URL of the person's avatar image.", null=True
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="url",
            field=models.URLField(
                blank=True, help_text="Website link for the person", null=True
            ),
        ),
    ]
