# Generated by Django 5.1.2 on 2024-10-21 18:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("podcast_analyzer", "0002_alter_person_img_url_alter_person_url"),
    ]

    operations = [
        migrations.AlterField(
            model_name="podcast",
            name="itunes_categories",
            field=models.ManyToManyField(
                blank=True,
                related_name="podcasts",
                to="podcast_analyzer.itunescategory",
            ),
        ),
    ]
