{% extends "podcast_analyzer/app_base.html" %}
{% load i18n %}
{% block extratitle %}{% blocktranslate with title=group.name %}Delete {{ title }}?{% endblocktranslate %} - {% endblock %}

{% block breadcrumbs %}

  <li><a href="{% url 'podcast_analyzer:ag-list' %}">{% translate "Analysis Groups" %}</a></li>
  <li><a href="{{ group.get_absolute_url }}">{{ group.name }}</a> </li>
  <li class='is-active'><a href="#" aria-current="page">{% translate "Delete" %}</a> </li>

{% endblock breadcrumbs %}

{% block content %}
  <h1>{% blocktranslate with title=group.name %}Delete {{ title }}?{% endblocktranslate %}</h1>
  <p><strong>{% translate "Are you sure you want to delete this analysis group?" %}</strong></p>
  <p>{% translate "This group is associated with the following records. (They will not be deleted.)" %}</p>
  <ul>
    <li><strong>{% translate "Podcasts" %}:</strong> {{ group.num_podcasts }}</li>
    <li><strong>{% translate "Episodes" %}:</strong> {{ group.num_episodes }}</li>
    <li><strong>{% translate "Seasons" %}:</strong> {{ group.num_seasons }}</li>
  </ul>
  <form method="post">
    {% csrf_token %}
    {{ form }}
    <div>
      <a href="{{ group.get_absolute_url }}">{% translate "Nevermind! Take me back." %}</a>
    </div>
    <div>
      <button type="submit">{% translate "Delete" %}</button>
    </div>
  </form>
{% endblock content %}
